# -*- python -*-
# $RCSfile: loggergui.py,v $
# $Revision: 1.5 $
# $Author: fyc $
# $Date: 2013-06-20 16:05:00 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@nist.gov. 

## This is a gui for gtklogger, to be used when recording log files.
## It must be run as a separate process, not as part of the program
## being instrumented.  The output from gtklogger should be piped to
## this program.  This program takes the name of the log file as a
## command line argument.

## The gui allows comments to be inserted into the log file as the log
## is recorded, making it easier to instrument it later. 
import gobject
import gtk
import sys
import gtklogger

starting = False
replaying = False

class GUI:
    def __init__(self):
        global logfile, logfilename
        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        window.set_title("gtklogger:" + logfilename)
        gtklogger.newTopLevelWidget(window, "gtklogger:" + logfilename)
        gtklogger.connect(window, "delete-event", self.quit)
        box = gtk.VBox()
        window.add(box)

        logscroll = gtk.ScrolledWindow()
        gtklogger.logScrollBars(logscroll, 'LogScroll')
        logscroll.set_shadow_type(gtk.SHADOW_IN)
        box.pack_start(logscroll, expand=True, fill=True)
        self.logtextview = gtk.TextView()
        gtklogger.setWidgetName(self.logtextview, "LogTextView")
        self.logtextview.set_editable(False)
        self.logtextview.set_cursor_visible(False)
        self.logtextview.set_wrap_mode(gtk.WRAP_WORD)
        logscroll.add(self.logtextview)

        commentbox = gtk.HBox()
        box.pack_start(commentbox, expand=False, fill=False)
        
        self.commentText = gtk.Entry()
        gtklogger.setWidgetName(self.commentText, 'CommentText')
        commentbox.pack_start(self.commentText, expand=True, fill=True)
        
        self.commentButton = gtk.Button("Comment")
        gtklogger.core.setWidgetName(self.commentButton, 'Comment')
        commentbox.pack_start(self.commentButton, expand=False, fill=False)
        gtklogger.core.connect(self.commentButton, 'clicked', self.commentCB)

        self.clearButton = gtk.Button('Clear')
        gtklogger.setWidgetName(self.clearButton, 'Clear')
        commentbox.pack_start(self.clearButton, expand=False, fill=False)
        gtklogger.connect(self.clearButton, 'clicked', self.clearCB)

        window.show_all()

    def addLine(self, line):
        buffer = self.logtextview.get_buffer()
        buffer.insert(buffer.get_end_iter(), line)
        mark = buffer.create_mark(None, buffer.get_end_iter())
        self.logtextview.scroll_mark_onscreen(mark)
        buffer.delete_mark(mark)
    
    def quit(self, *args):
        global starting, replaying
        gtk.main_quit()
        if starting == True or replaying == True:
            starting = False
            replaying = False
            gtklogger.stop()
        logfile.close()
    
    def clearCB(self, button):
        self.commentText.set_text("")

    def commentCB(self, button):
        global logfile
        text = '# ' + self.commentText.get_text()
        print >> logfile, text
        self.addLine(text+'\n')

inbuf = ""
def inputhandler(source, condition):
    global inbuf
    line = sys.stdin.readline()
    if not line:
        gui.quit()
        return False
    if line[-1] == '\n':
        fullline = inbuf + line
        gui.addLine(fullline)
        print >> logfile, fullline,
        logfile.flush()
        inbuf = ""
    else:
        inbuf += line
    return True

def start(name):
    global logfilename, logfile, gui
    logfilename = name
    logfile = file(logfilename, "w")
    gobject.io_add_watch(sys.stdin, gobject.IO_IN, inputhandler)
    if starting == True:
        gtklogger.start(logfile,True,False)
    gui = GUI()
    gtk.main()
    
if __name__ == "__main__":
    gtklogger.logutils.set_debugLevel(3)
    if sys.argv[1] == "--autorecord":
        starting = True
        start(sys.argv[2])
    elif sys.argv[1] == "--autoreplay":
        replaying = True
        gtklogger.replay(sys.argv[2],None,None,None,True,False)
        start(sys.argv[2])
    else:
        start(sys.argv[1])
